from typing import Any

BOOL_TRUEISH = ["1", "yes", "y", "t", "true", "on", "enabled"]


def as_bool(value: Any, default: bool = False) -> bool:
    if isinstance(value, bool):
        return value
    if value is None:
        return default
    value = str(value).strip().lower()
    if not len(value):
        return default
    return value in BOOL_TRUEISH



from typing import Any, Dict, List, TypeVar
from collections.abc import Mapping

from banal.lists import is_sequence, ensure_list

K = TypeVar("K")
V = TypeVar("V")


def is_mapping(obj: Any) -> bool:
    return isinstance(obj, Mapping)


def ensure_dict(obj: Any) -> Dict[K, V]:
    if is_mapping(obj) or hasattr(obj, "items"):
        return dict(obj.items())
    return {}


def clean_dict(data: Any) -> Any:
    """Remove None-valued keys from a dictionary, recursively."""
    if isinstance(data, Mapping):
        out = {}
        for k, v in data.items():
            if v is not None:
                out[k] = clean_dict(v)
        return out
    elif is_sequence(data):
        return [clean_dict(d) for d in data if d is not None]
    return data


def keys_values(data: Dict[K, V], *keys: K) -> List[V]:
    """Get an entry as a list from a dict. Provide a fallback key."""
    values: List[V] = []
    if isinstance(data, Mapping):
        for key in keys:
            if key in data:
                values.extend(ensure_list(data[key]))
    return values



import types
from hashlib import sha1
from itertools import chain
from typing import Any, Union, Iterable
from datetime import date, datetime

from banal.dicts import is_mapping
from banal.lists import is_sequence


def _bytes_str(obj: Union[str, bytes]) -> bytes:
    if not isinstance(obj, str):
        return obj
    return obj.encode("utf-8")


def bytes_iter(obj: Any) -> Iterable[bytes]:
    """Turn a complex object into an iterator of byte strings.
    The resulting iterator can be used for caching.
    """
    if obj is None:
        return
    elif isinstance(obj, (bytes, str)):
        yield _bytes_str(obj)
    elif isinstance(obj, (date, datetime)):
        yield _bytes_str(obj.isoformat())
    elif is_mapping(obj):
        if None in obj:
            yield from bytes_iter(obj.pop(None))
        for key in sorted(obj.keys()):
            for out in chain(bytes_iter(key), bytes_iter(obj[key])):
                yield out
    elif is_sequence(obj):
        if isinstance(obj, (list, set)):
            try:
                obj = sorted(obj)
            except Exception:
                pass
        for item in obj:
            for out in bytes_iter(item):
                yield out
    elif isinstance(
        obj,
        (
            types.FunctionType,
            types.BuiltinFunctionType,
            types.MethodType,
            types.BuiltinMethodType,
        ),
    ):
        yield _bytes_str(getattr(obj, "func_name", ""))
    else:
        yield _bytes_str(str(obj))


def hash_data(obj: Any) -> str:
    """Generate a SHA1 from a complex object."""
    collect = sha1()
    for data in bytes_iter(obj):
        collect.update(data)
    return collect.hexdigest()



import sys
from typing import Optional


def decode_path(file_path: Optional[str]) -> Optional[str]:
    """Turn a path name into unicode."""
    if file_path is None:
        return None
    if isinstance(file_path, bytes):
        file_path = file_path.decode(sys.getfilesystemencoding())
    return file_path



from typing import cast, overload, Optional, List, Set, Any, TypeVar
from typing import Generator, Sequence, Iterable

T = TypeVar("T")


def is_sequence(obj: Any) -> bool:
    return isinstance(obj, Sequence) and not isinstance(obj, (str, bytes))


def is_listish(obj: Any) -> bool:
    """Check if something quacks like a list."""
    if isinstance(obj, (list, tuple, set)):
        return True
    return is_sequence(obj)


def unique_list(lst: Sequence[T]) -> List[T]:
    """Make a list unique, retaining order of initial appearance."""
    uniq = []
    for item in lst:
        if item not in uniq:
            uniq.append(item)
    return uniq


@overload
def ensure_list(obj: None) -> List[None]:
    pass


@overload
def ensure_list(obj: Iterable[T]) -> List[T]:
    pass


@overload
def ensure_list(obj: T) -> List[T]:
    pass


def ensure_list(obj: Any) -> List[T]:
    """Make the returned object a list, otherwise wrap as single item."""
    if obj is None:
        return []
    if is_listish(obj):
        return [o for o in cast(Sequence[T], obj)]
    return [cast(T, obj)]


def chunked_iter(
    iterable: Iterable[T], batch_size: int = 500
) -> Generator[List[T], None, None]:
    """Pick `batch_size` items from an iterable and treat them as a batch list."""
    batch = list()
    for item in iterable:
        batch.append(item)
        if len(batch) >= batch_size:
            yield batch
            batch = list()
    if len(batch) > 0:
        yield batch


def chunked_iter_sets(
    iterable: Iterable[T], batch_size: int = 500
) -> Generator[Set[T], None, None]:
    """Pick `batch_size` items from an iterable and treat them as a batch set."""
    batch = set()
    for item in iterable:
        batch.add(item)
        if len(batch) >= batch_size:
            yield batch
            batch = set()
    if len(batch) > 0:
        yield batch


def first(lst: Sequence[T]) -> Optional[T]:
    """Return the first non-null element in the list, or None."""
    item: T
    for item in ensure_list(lst):
        if item is not None:
            return item
    return None



from banal.lists import is_sequence, is_listish
from banal.lists import ensure_list, unique_list
from banal.lists import first, chunked_iter, chunked_iter_sets
from banal.dicts import is_mapping, clean_dict
from banal.dicts import ensure_dict, keys_values
from banal.filesystem import decode_path
from banal.cache import hash_data
from banal.bools import as_bool

__all__ = [
    "is_sequence",
    "is_listish",
    "ensure_list",
    "unique_list",
    "chunked_iter",
    "chunked_iter_sets",
    "first",
    "as_bool",
    "is_mapping",
    "clean_dict",
    "ensure_dict",
    "keys_values",
    "decode_path",
    "hash_data",
]



from setuptools import setup  # type: ignore

with open("README.md") as f:
    long_description = f.read()


setup(
    name="banal",
    version="1.0.6",
    description="Commons of banal micro-functions for Python.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    keywords="utilities commons functions",
    author="Friedrich Lindenberg",
    author_email="friedrich@pudo.org",
    url="http://github.com/pudo/banal",
    license="MIT",
    namespace_packages=[],
    package_data={"banal": ["py.typed"]},
    packages=["banal"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[],
    extras_require={
        "dev": [
            "mypy",
            "wheel",
        ]
    },
)
